<?php

namespace App\Http\Middleware;

use Closure;
use Illuminate\Http\Request;
use Tymon\JWTAuth\Facades\JWTAuth;
use Symfony\Component\HttpKernel\Exception\UnauthorizedHttpException;
use Tymon\JWTAuth\Exceptions\TokenInvalidException;
use Carbon\Carbon;

class JwtMiddleware
{
    /**
     * Handle an incoming request.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \Closure(\Illuminate\Http\Request): (\Illuminate\Http\Response|\Illuminate\Http\RedirectResponse)  $next
     * @return \Illuminate\Http\Response|\Illuminate\Http\RedirectResponse
     */
    public function handle(Request $request, Closure $next)
    {

        try {
            $token = JWTAuth::parseToken();
            if (!$token->getPayload()) {
                throw new TokenInvalidException('Token not provided');
            }
            $user = $token->authenticate();

            $userPayload = $token->getPayload();

            // Attach decoded token payload data to the request so all other controler can access this data
            $request->attributes->add(['userJwtPayload' => $userPayload]);


            $loggoutErr = "You are logged out due to security reason!";
            $needPinCodeErr = "Due to security reason you need to enter the pin code!";

            // ------- Validate if the tokenGenTime is passed 1 hour then return the fail with the needPinCode variable so the front end will be able no know that we need to refresh the token by the pincode

            // Parse the datetime string into a Carbon instance
            $requestDateTime = Carbon::parse($userPayload["tokenGenTime"]);

            // Check if sepcific time passed
            $isTimePassed = $requestDateTime->diffInHours(Carbon::now()) >= 1;

            // If 6 hours passed
            $ifLongTimePassed = $requestDateTime->diffInHours(Carbon::now()) >= 6;
            

            // If long time passed then signout
            if ($ifLongTimePassed) {
                return response()->json(['loggedOut' => true, 'status' => 0, 'error' => $loggoutErr]);
            }

            // If time passed
            if ($isTimePassed) {
                return response()->json(['needPinCode' => true, 'status' => 0, 'error' => $needPinCodeErr]);
            }

            // --------------------------


            // --------- Validate if the userAgent changed then request for the pin ------
            if ($userPayload["user_agent"] !== $request->userAgent()) {
                return response()->json(['needPinCode' => true, 'status' => 0, 'error' => $needPinCodeErr]);
            }
            // ---------------------------------------------------------------------------


            // --------- Validate if the ip did not matched -------------
            if ($userPayload["ip"] !== $request->ip()) {

                // If we want to session out on ip change then do it else we req the pin code
                if ($userPayload["signoutOnIpChange"]) {
                    return response()->json(['loggedOut' => true, 'status' => 0, 'error' => $loggoutErr]);
                }

                // We are here it means we want the user to enter the pincode
                return response()->json(['needPinCode' => true, 'status' => 0, 'error' => $needPinCodeErr]);
            }
            // ----------------------------------------------------------


            //return response()->json(['payLoad' => $userPayload, 'user' => $user]);
        } catch (TokenInvalidException $e) {
            return response()->json(['error' => $e->getMessage()], 401);
        } catch (UnauthorizedHttpException $e) {
            return response()->json(['error' => 'Token not provided'], 401);
        }

        return $next($request);
    }
}
